package mpu

import (
	"fmt"
	"io"

	"github.com/CGSG-2021-AE4/mpu-test/pkg/mpu6050"
)

type MpuData struct {
	TempData  float32    `json:"Temperature"`
	AccelData [3]float32 `json:"Accelerometer"`
	GyroData  [3]float32 `json:"Gyroscope"`
}

type mpu struct {
	device mpu6050.Device

	data MpuData
}

type Mpu interface {
	Init() error
	Update() error
	Validate() bool
	GetData() MpuData
	io.Closer
}

func New() Mpu {
	return &mpu{
		device: nil,
	}
}

func (m *mpu) Init() error {
	d, err := mpu6050.Open()
	if err != nil {
		return fmt.Errorf("open mpu connection: %w", err)
	}
	m.device = d
	return nil
}

func (m *mpu) Update() (err error) {
	m.data.TempData, err = m.device.GetTemp()
	if err != nil {
		return fmt.Errorf("get temp data: %w", err)
	}
	m.data.AccelData, err = m.device.GetAccel(false)
	if err != nil {
		return fmt.Errorf("get accel data: %w", err)
	}
	m.data.GyroData, err = m.device.GetGyro()
	if err != nil {
		return fmt.Errorf("get gyro data: %w", err)
	}
	return nil
}

func (m *mpu) Validate() bool {
	return m.device != nil
}

func (m *mpu) GetData() MpuData {
	return m.data
}

func (m *mpu) Close() error {
	defer func() {
		m.device = nil
	}()
	return m.device.Close()
}
